package com.syntevo.plugin.tagmultiple;

import java.awt.*;
import java.awt.event.*;
import java.util.List;
import javax.swing.*;
import javax.swing.text.*;

import com.syntevo.openapi.gui.*;
import com.syntevo.openapi.gui.dialog.*;
import com.syntevo.q.gui.layout.*;

/**
 * @author syntevo GmbH
 */
final class TagMultipleProjectRootsDialog extends AbstractOkCancelDialog {

	// Fields =================================================================

	private final JComboBox tagNameComboBox;
	private final JTextField comboBoxTextField;
	private final JCheckBox fixExternalsCheckBox;
	private final JTextArea commitMessageTextArea;
	private final GuiSpacings spacings;

	// Setup ==================================================================

	public TagMultipleProjectRootsDialog(List previousTags, boolean fixExternals, String initialCommitMessage, GuiSpacings spacings) {
		this.spacings = spacings;

		tagNameComboBox = new JComboBox(previousTags.toArray());
		tagNameComboBox.setEditable(true);

		comboBoxTextField = ((JTextField)tagNameComboBox.getEditor().getEditorComponent());
		comboBoxTextField.selectAll();
		comboBoxTextField.setColumns(20);
		selectTextIfTabbingToComponent(comboBoxTextField);
		fixComponentSize(tagNameComboBox);

		fixExternalsCheckBox = new JCheckBox("Fix external revisions");
		fixExternalsCheckBox.setSelected(fixExternals);
		fixExternalsCheckBox.setMnemonic('F');

		commitMessageTextArea = new JTextArea(5, 30);
		if (initialCommitMessage != null) {
			commitMessageTextArea.setText(initialCommitMessage);
		}
		selectTextIfTabbingToComponent(commitMessageTextArea);
	}

	// Implemented ============================================================

	@Override
	public String getTitle() {
		return "Tag Multiple";
	}

	@Override
	public JComponent createComponent(Container parent) {
		final JComponent panel = new JPanel();
		parent.add(panel);

		final JScrollPane commitMessageScrollPane = new JScrollPane(commitMessageTextArea);
		fixComponentSize(commitMessageScrollPane);

		final TableGridLayoutFactory layout = new TableGridLayoutFactory(panel);
		layout.addColumnNoStretch();
		layout.addColumnNoStretch(spacings.getXRelated());
		layout.addColumnStretchOnResize();
		layout.addRowNoStretch();
		layout.addRowNoStretch(spacings.getYUnrelated());
		layout.addRowNoStretch();
		layout.addRowNoStretch(spacings.getYUnrelated());
		layout.addRowStretchOnResize();
		layout.addRowNoStretch(spacings.getYUnrelated());
		layout.addRowNoStretch();
		layout.addComponent(createLabelFor("Tag Name", 'T', tagNameComboBox), 0, 0);
		layout.addComponent(tagNameComboBox, 0, 2);
		layout.addComponent(createLabelFor("Commit Message", 'M', commitMessageTextArea), 2, 0, 1, 3);
		layout.addComponent(commitMessageScrollPane, 4, 0, 1, 3);
		layout.addComponent(fixExternalsCheckBox, 6, 0, 1, 3, true, true, true, false);
		return panel;
	}

	@Override
	protected void validate() throws DialogValidationFailedException {
		if (getTagName() == null) {
			throw new DialogValidationFailedException("Please enter a tag name.", tagNameComboBox);
		}
	}

	// Accessing ==============================================================

	public String getTagName() {
		final String text = comboBoxTextField.getText();
		return text != null && text.trim().length() == 0 ? null : text;
	}

	public boolean isFixExternals() {
		return fixExternalsCheckBox.isSelected();
	}

	public String getCommitMessage() {
		return commitMessageTextArea.getText();
	}

	// Utils ==================================================================

	private static void selectTextIfTabbingToComponent(final JTextComponent textComponent) {
		textComponent.selectAll();
		textComponent.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost(FocusEvent e) {
				textComponent.selectAll();
			}
		});
	}

	private static void fixComponentSize(JComponent component) {
		final Dimension size = component.getPreferredSize();
		component.setMinimumSize(size);
		component.setPreferredSize(size);
	}

	private static JLabel createLabelFor(String text, char mnemonic, JComponent component) {
		final JLabel label = new JLabel(text + ':');
		label.setLabelFor(component);
		label.setDisplayedMnemonic(mnemonic);
		return label;
	}
}
